# Generated by roxygen2: do not edit by hand

export("assayColorMap<-")
export("colDataColorMap<-")
export("rowDataColorMap<-")
export(.activateAppOptionRegistry)
export(.addCustomLabelsCommands)
export(.addFacets)
export(.addLabelCentersCommands)
export(.addMultiSelectionPlotCommands)
export(.addSpecificTour)
export(.addTourStep)
export(.allowableChoiceError)
export(.allowableColorByDataChoices)
export(.allowableXAxisChoices)
export(.allowableYAxisChoices)
export(.buildAes)
export(.buildLabs)
export(.cacheCommonInfo)
export(.checkboxGroupInput.iSEE)
export(.checkboxInput.iSEE)
export(.clearSpecificTours)
export(.colorByNoneDotPlotField)
export(.colorByNoneDotPlotScale)
export(.conditionalOnCheckGroup)
export(.conditionalOnCheckSolo)
export(.conditionalOnRadio)
export(.createCustomDimnamesModalObservers)
export(.createObservers)
export(.createProtectedParameterObservers)
export(.createUnprotectedParameterObservers)
export(.dataParamBoxOpen)
export(.deactivateAppOptionRegistry)
export(.defineDataInterface)
export(.defineInterface)
export(.defineOutput)
export(.definePanelTour)
export(.defineVisualColorInterface)
export(.defineVisualFacetInterface)
export(.defineVisualOtherInterface)
export(.defineVisualPointInterface)
export(.defineVisualShapeInterface)
export(.defineVisualSizeInterface)
export(.defineVisualTextInterface)
export(.emptyDefault)
export(.exportOutput)
export(.extractAssaySubmatrix)
export(.findAtomicFields)
export(.fullName)
export(.generateDotPlot)
export(.generateDotPlotData)
export(.generateOutput)
export(.generateTable)
export(.getCachedCommonInfo)
export(.getDotPlotColorHelp)
export(.getEncodedName)
export(.getFullName)
export(.getPanelColor)
export(.getSpecificTours)
export(.getTooltipUI)
export(.hideInterface)
export(.isAssayNumeric)
export(.isBrushable)
export(.multiSelectHistory)
export(.multiSelectionActive)
export(.multiSelectionAvailable)
export(.multiSelectionClear)
export(.multiSelectionCommands)
export(.multiSelectionDimension)
export(.multiSelectionInvalidated)
export(.multiSelectionResponsive)
export(.multiSelectionRestricted)
export(.multipleChoiceError)
export(.noSelection)
export(.numericInput.iSEE)
export(.organizationHeight)
export(.organizationWidth)
export(.panelColor)
export(.prioritizeDotPlotData)
export(.processMultiSelections)
export(.radioButtons.iSEE)
export(.refineParameters)
export(.removeInvalidChoices)
export(.renderOutput)
export(.replaceMissingWithFirst)
export(.requestActiveSelectionUpdate)
export(.requestCleanUpdate)
export(.requestUpdate)
export(.retrieveOutput)
export(.selectInput.iSEE)
export(.selectInputHidden)
export(.selectizeInput.iSEE)
export(.setCachedCommonInfo)
export(.showSelectionDetails)
export(.singleSelectionDimension)
export(.singleSelectionSlots)
export(.singleSelectionValue)
export(.singleStringError)
export(.sliderInput.iSEE)
export(.textEval)
export(.trackMultiSelection)
export(.trackRelinkedSelection)
export(.trackSingleSelection)
export(.trackUpdate)
export(.validLogicalError)
export(.validNumberError)
export(.validStringError)
export(.whichGroupable)
export(.whichNumeric)
export(ColumnDataPlot)
export(ColumnDataTable)
export(ComplexHeatmapPlot)
export(ExperimentColorMap)
export(FeatureAssayPlot)
export(ReducedDimensionPlot)
export(RowDataPlot)
export(RowDataTable)
export(SampleAssayPlot)
export(assayColorMap)
export(checkColormapCompatibility)
export(cleanDataset)
export(colDataColorMap)
export(collapseBox)
export(columnSelectionColorMap)
export(createCustomPlot)
export(createCustomTable)
export(createLandingPage)
export(defaultTour)
export(filterDT)
export(filterDTColumn)
export(getAllAppOptions)
export(getAppOption)
export(getPanelDefault)
export(iSEE)
export(iSEEOptions)
export(jitterSquarePoints)
export(jitterViolinPoints)
export(lassoPoints)
export(multiSelectionToFactor)
export(panelDefaults)
export(registerAppOptions)
export(rowDataColorMap)
export(rowSelectionColorMap)
export(subsetPointsByGrid)
export(synchronizeAssays)
exportClasses(ColumnDataPlot)
exportClasses(ColumnDataTable)
exportClasses(ColumnDotPlot)
exportClasses(ColumnTable)
exportClasses(ComplexHeatmapPlot)
exportClasses(DotPlot)
exportClasses(ExperimentColorMap)
exportClasses(FeatureAssayPlot)
exportClasses(Panel)
exportClasses(ReducedDimensionPlot)
exportClasses(RowDataPlot)
exportClasses(RowDataTable)
exportClasses(RowDotPlot)
exportClasses(RowTable)
exportClasses(SampleAssayPlot)
exportClasses(Table)
exportMethods("[[")
exportMethods("[[<-")
exportMethods(.allowableColorByDataChoices)
exportMethods(.allowableXAxisChoices)
exportMethods(.allowableYAxisChoices)
exportMethods(.cacheCommonInfo)
exportMethods(.colorByNoneDotPlotField)
exportMethods(.colorByNoneDotPlotScale)
exportMethods(.createObservers)
exportMethods(.defineDataInterface)
exportMethods(.defineInterface)
exportMethods(.defineOutput)
exportMethods(.definePanelTour)
exportMethods(.defineVisualColorInterface)
exportMethods(.defineVisualFacetInterface)
exportMethods(.defineVisualOtherInterface)
exportMethods(.defineVisualPointInterface)
exportMethods(.defineVisualShapeInterface)
exportMethods(.defineVisualSizeInterface)
exportMethods(.defineVisualTextInterface)
exportMethods(.exportOutput)
exportMethods(.fullName)
exportMethods(.generateDotPlot)
exportMethods(.generateDotPlotData)
exportMethods(.generateOutput)
exportMethods(.generateTable)
exportMethods(.hideInterface)
exportMethods(.isBrushable)
exportMethods(.multiSelectionActive)
exportMethods(.multiSelectionAvailable)
exportMethods(.multiSelectionClear)
exportMethods(.multiSelectionCommands)
exportMethods(.multiSelectionDimension)
exportMethods(.multiSelectionInvalidated)
exportMethods(.multiSelectionResponsive)
exportMethods(.multiSelectionRestricted)
exportMethods(.panelColor)
exportMethods(.prioritizeDotPlotData)
exportMethods(.refineParameters)
exportMethods(.renderOutput)
exportMethods(.showSelectionDetails)
exportMethods(.singleSelectionDimension)
exportMethods(.singleSelectionSlots)
exportMethods(.singleSelectionValue)
exportMethods(cleanDataset)
exportMethods(initialize)
exportMethods(show)
exportMethods(updateObject)
import(SingleCellExperiment)
import(SummarizedExperiment)
import(methods)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(BiocGenerics,match)
importFrom(BiocGenerics,updateObject)
importFrom(ComplexHeatmap,Heatmap)
importFrom(ComplexHeatmap,columnAnnotation)
importFrom(ComplexHeatmap,draw)
importFrom(ComplexHeatmap,rowAnnotation)
importFrom(DT,dataTableOutput)
importFrom(DT,dataTableProxy)
importFrom(DT,datatable)
importFrom(DT,renderDataTable)
importFrom(DT,selectRows)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,setValidity2)
importFrom(SingleCellExperiment,"reducedDimNames<-")
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SummarizedExperiment,"assayNames<-")
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(SummarizedExperiment,rowRanges)
importFrom(circlize,colorRamp2)
importFrom(colourpicker,colourInput)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_blank)
importFrom(ggplot2,geom_density_2d)
importFrom(ggplot2,geom_path)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_gradientn)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_void)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.off)
importFrom(grDevices,hcl)
importFrom(grDevices,pdf)
importFrom(grDevices,rgb)
importFrom(graphics,plot)
importFrom(grid,gpar)
importFrom(grid,unit)
importFrom(igraph,"E<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,add_edges)
importFrom(igraph,adjacent_vertices)
importFrom(igraph,degree)
importFrom(igraph,delete_edges)
importFrom(igraph,delete_vertices)
importFrom(igraph,get.edge.ids)
importFrom(igraph,is_dag)
importFrom(igraph,make_graph)
importFrom(igraph,simplify)
importFrom(igraph,topo_sort)
importFrom(listviewer,jsonedit)
importFrom(listviewer,jsoneditOutput)
importFrom(listviewer,renderJsonedit)
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,new)
importFrom(mgcv,in.out)
importFrom(rintrojs,introjs)
importFrom(rintrojs,introjsUI)
importFrom(shiny,HTML)
importFrom(shiny,a)
importFrom(shiny,actionButton)
importFrom(shiny,addResourcePath)
importFrom(shiny,br)
importFrom(shiny,brushOpts)
importFrom(shiny,brushedPoints)
importFrom(shiny,checkboxGroupInput)
importFrom(shiny,checkboxInput)
importFrom(shiny,code)
importFrom(shiny,column)
importFrom(shiny,conditionalPanel)
importFrom(shiny,div)
importFrom(shiny,downloadButton)
importFrom(shiny,downloadHandler)
importFrom(shiny,em)
importFrom(shiny,eventReactive)
importFrom(shiny,fileInput)
importFrom(shiny,fluidRow)
importFrom(shiny,getDefaultReactiveDomain)
importFrom(shiny,h4)
importFrom(shiny,hoverOpts)
importFrom(shiny,hr)
importFrom(shiny,icon)
importFrom(shiny,incProgress)
importFrom(shiny,includeCSS)
importFrom(shiny,includeScript)
importFrom(shiny,insertUI)
importFrom(shiny,isolate)
importFrom(shiny,modalDialog)
importFrom(shiny,nearPoints)
importFrom(shiny,numericInput)
importFrom(shiny,observe)
importFrom(shiny,observeEvent)
importFrom(shiny,onFlushed)
importFrom(shiny,onStop)
importFrom(shiny,p)
importFrom(shiny,plotOutput)
importFrom(shiny,pre)
importFrom(shiny,radioButtons)
importFrom(shiny,reactiveValues)
importFrom(shiny,removeModal)
importFrom(shiny,removeNotification)
importFrom(shiny,removeUI)
importFrom(shiny,renderPlot)
importFrom(shiny,renderUI)
importFrom(shiny,req)
importFrom(shiny,selectInput)
importFrom(shiny,selectizeInput)
importFrom(shiny,shinyApp)
importFrom(shiny,showModal)
importFrom(shiny,showNotification)
importFrom(shiny,singleton)
importFrom(shiny,sliderInput)
importFrom(shiny,span)
importFrom(shiny,stopApp)
importFrom(shiny,strong)
importFrom(shiny,tagList)
importFrom(shiny,tags)
importFrom(shiny,textInput)
importFrom(shiny,uiOutput)
importFrom(shiny,updateActionButton)
importFrom(shiny,updateCheckboxGroupInput)
importFrom(shiny,updateNumericInput)
importFrom(shiny,updateSelectInput)
importFrom(shiny,updateSelectizeInput)
importFrom(shiny,wellPanel)
importFrom(shiny,withProgress)
importFrom(shinyAce,aceEditor)
importFrom(shinyAce,updateAceEditor)
importFrom(shinyWidgets,addSpinner)
importFrom(shinydashboard,box)
importFrom(shinydashboard,dashboardBody)
importFrom(shinydashboard,dashboardHeader)
importFrom(shinydashboard,dashboardPage)
importFrom(shinydashboard,dashboardSidebar)
importFrom(shinydashboard,dropdownMenu)
importFrom(shinydashboard,menuItem)
importFrom(shinydashboard,notificationItem)
importFrom(shinydashboard,tabBox)
importFrom(shinydashboard,valueBox)
importFrom(shinydashboard,valueBoxOutput)
importFrom(shinyjs,disable)
importFrom(shinyjs,disabled)
importFrom(shinyjs,enable)
importFrom(shinyjs,hidden)
importFrom(shinyjs,useShinyjs)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(utils,adist)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,citation)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,read.delim)
importFrom(utils,sessionInfo)
importFrom(utils,write.csv)
importFrom(utils,zip)
importFrom(vipor,offsetX)
importFrom(viridisLite,viridis)
importMethodsFrom(BiocGenerics,ncol)
importMethodsFrom(BiocGenerics,nrow)
importMethodsFrom(SummarizedExperiment,assayNames)
importMethodsFrom(SummarizedExperiment,assays)
